import HarmonyDataProvider from "../DataProviders/HarmonyDataProvider";
import { IDeviceButton } from "../Models/Config";
import { IAccessory } from "./IAccessory";
import { ICommand } from "../Models";
import { HarmonyDevice } from "../Models/HarmonyDevice";

let Service: HAPNodeJS.Service;
let Characteristic: HAPNodeJS.Characteristic;

export interface IDeviceButtonProps {
    dataProvider: HarmonyDataProvider,
    buttonName: string,
    displayName: string,
    deviceInfo: IDeviceButton,
    api: any,
    log: any,
    homebridge: any,
}

export class DeviceButton implements IAccessory {
    private _api: any;
    private _homebridge: any;
    private _log: any = {};

    //Service fields
    private _switchService: HAPNodeJS.Service;
    private _infoService: HAPNodeJS.Service;

    private _buttonInfo: IDeviceButton;

    private _dataProvider: HarmonyDataProvider;

    private _device!: HarmonyDevice;

    private _buttonState: boolean;

    constructor(props: IDeviceButtonProps) {
        //Assign class variables
        this._log = props.log;
        this._api = props.api;
        Service = props.api.hap.Service;
        Characteristic = props.api.hap.Characteristic;
        this.name = props.displayName;
        this._homebridge = props.homebridge;

        this._buttonInfo = props.deviceInfo;

        this._dataProvider = props.dataProvider;

        this._buttonState = false;

        if (this._buttonInfo.NumberOfKeyPresses && this._buttonInfo.IsStateful) {
            throw new Error("A button cannot be stateful and be pressed more than once");
        }

        //Assign default number of key presses
        if (!this._buttonInfo.NumberOfKeyPresses) {
            this._buttonInfo.NumberOfKeyPresses = 1;
        }

        this.platformAccessory = new this._homebridge.platformAccessory(this.name, this.generateUUID(), this._homebridge.hap.Accessory.Categories.SWITCH);

        //@ts-ignore
        this._infoService = new Service.AccessoryInformation();
        this._infoService.setCharacteristic(Characteristic.Manufacturer, "Brandon Watson")
        this._infoService.setCharacteristic(Characteristic.Model, "Device Button")
        this._infoService.setCharacteristic(Characteristic.SerialNumber, "123-456-789");

        this._switchService = new Service.Switch(
            this.name,
            'switchService'
        )

        this._switchService.getCharacteristic(Characteristic.On)
            //@ts-ignore
            .on("set", this.onSwitchSet)
            .updateValue(this._buttonState)
            .on("get", this.onSwitchGet);
    }

    /**
     * Required by homebridge.
     */
    public name: string;

    public platformAccessory: any;

    /**
     * Called by homebridge to gather services.
     */
    public getServices = (): Array<HAPNodeJS.Service> => {
        return [this._infoService, this._switchService!];
    }

    /**
     * Handler for switch set event
     * @param callback The callback function to call when complete
     */
    private onSwitchSet = async (newState: boolean, callback: (error?: Error | null | undefined) => void) => {
        if (!this._buttonInfo.IsStateful && newState === this._buttonState) {
            return callback();
        }

        //Get device command if we don't have it
        if (!this._device) {
            this._device = this._dataProvider.getDeviceFromName(this._buttonInfo.DeviceName);
        }

        //Execute command
        if (this._device) {
            //change state if stateful
            if (this._buttonInfo.IsStateful && this._buttonState != newState) {
                this._buttonState = newState
                await this._device.sendCommand(this._buttonInfo.ButtonName);
            } else if (!this._buttonInfo.IsStateful) {
                //Send the number of configured key presses
                for (let i = 0; i < this._buttonInfo.NumberOfKeyPresses; i++) {
                    await this._device.sendCommand(this._buttonInfo.ButtonName);
                }
                this._switchService.getCharacteristic(Characteristic.On).updateValue(false);
                return callback(new Error("Normal Response"));
            }
        }
        return callback();
    }

    /**
     * Handler for switch get event
     * @param callback The callback function to call when complete
     */
    private onSwitchGet = (callback: (error: Error | null, value: boolean) => void) => {
        //Only return state if button is stateful
        if (this._buttonInfo.IsStateful) {
            return callback(null, this._buttonState);
        } else {
            return callback(null, false)
        }

    }

    /**
     * Helper function to generate a UUID
     */
    private generateUUID(): string { // Public Domain/MIT
        var d = new Date().getTime();
        if (typeof performance !== 'undefined' && typeof performance.now === 'function') {
            d += performance.now(); //use high-precision timer if available
        }
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
        });
    }
}